@@ -570,5 +570,9 @@ Connector.prototype = {
     //  "new='" + flaglist + "' :" + ( this.metaData["flags"] == flaglist) + "\n" );
     this.metaData["flags"] = flaglist;
   },
-
+  useSessionTestValues: function () {
+    if (typeof this.properties.testSessionJSON === 'string') {
+      this.data.session = JSON.parse(this.properties.testSessionJSON);
+    }
+  },
 };
