@@ -80,7 +80,7 @@ var jsonPathHelper = {
     let result = path
       ? jsonPath(container, path)
       : [container];
-    if (result === false) return false;
+    if (result === false) return null;
     let value = result[0];
     if (Array.isArray(value)) {
       if (key) {
@@ -127,7 +127,7 @@ var jsonPathHelper = {
       }
       return results;
     } else {
-      return false;
+      return null;
     }
   },
 
@@ -214,10 +214,15 @@ var jsonPathHelper = {
   //
   // src.path and dst.path must yield the same number of objects.
   //
-  // src.key is optional, src may yield scalars which will be wrapped in arrays
+  // src.key is optional
+  //
+  // src may yield scalars which will be wrapped in arrays
   //
   // dst.key is not optional as Javascript has no scalar references
   //
+  // if src.key is "*" will ignore dst.key and instead map every key in src to
+  // the same key in dst, applying func() to each.
+  //
   // If func() returns "##NOOP##" for an object, set() is not called and the dst
   // object is unchanged. NB: if func() returns undefined set() will delete
   // the target if it is to replace it.
@@ -226,7 +231,7 @@ var jsonPathHelper = {
       ? jsonPath(container, src.path)
       : [container];
 
-    if (srcObjs === false) logger.warn("source path " + src.path + " has no match");
+    if (srcObjs === undefined) logger.warn("source path " + src.path + " has no match");
 
     if (typeof(dst) !== "object") {
       var dst = src;
@@ -238,7 +243,7 @@ var jsonPathHelper = {
       var dstObjs = dst.path
         ? jsonPath(container, dst.path)
         : [container];
-      if (dstObjs === false) throw new Error("destination path " + dst.path + " has no match");
+      if (dstObjs === undefined) throw new Error("destination path " + dst.path + " has no match");
       else if (srcObjs.length !== dstObjs.length) {
         throw new Error(
           "Cannot map between JSONpaths that return differing numbers of objects:\n"
@@ -250,16 +255,21 @@ var jsonPathHelper = {
 
     for (let i = 0; i < srcObjs.length; i++) {
       let dstObj = dstObjs[i];
-      let dstKey = dst.key;
-      // src.key is optional
-      let srcArr;
-      if (!src.key) srcArr = srcObjs[i];
-      else srcArr = srcObjs[i][src.key];
-      if ((typeof srcArr !== 'undefined') && (!Array.isArray(srcArr))) srcArr = [srcArr];
-      var transformed = func(srcArr);
-      // noop on false return
-      if (transformed === "##NOOP##") continue;
-      else jsonPathHelper.addToArrayProp(transformed, dstObj, dstKey, dst.append);
+      // build an object to map all keys from through func() 
+      // and onto keys of the same name in dstObj. facilitates
+      // key wildcard and other potential extensions
+      let srcObj = {};
+      // without src.key we're taking this for an array of scalars
+      if (!src.key) srcObj[dst.key] = srcObjs[i];
+      else if (src.key === '*') srcObj = srcObjs[i];
+      else srcObj[dst.key] = srcObjs[i][src.key];
+      for (dstKey in srcObj) {
+        srcArr = srcObj[dstKey];
+        if ((typeof srcArr !== 'undefined') && (!Array.isArray(srcArr))) srcArr = [srcArr];
+        let transformed = func(srcArr);
+        if (transformed !== "##NOOP##")
+        jsonPathHelper.addToArrayProp(transformed, dstObj, dstKey, dst.append);
+      }
     }
   },
 
@@ -479,7 +489,7 @@ var jsonPathHelper = {
       label: "no array",
       spec: {path:"$.results[*].item", key:"shenanigans"},
       obj: {},
-      expected: false,
+      expected: null,
     },
     ];
 
@@ -499,7 +509,6 @@ var jsonPathHelper = {
         logger.error(test.expected);
         logger.error("Actual:");
         logger.error(JSON.stringify(result));
-        dump (result);
         return false;
       }
     }
@@ -624,7 +633,6 @@ var jsonPathHelper = {
       expected: '{}',
     },
 
-
     ];
 
     for (let i = 0; i < tests.length; i++) {
@@ -699,6 +707,19 @@ var jsonPathHelper = {
       expected: '{"results":[{"author":["billy","bob","BILLY","BOB"],"title":["a book"],"item":[{"location":["a library"]},{"location":["my house"]}]},{"author":["jill","JILL"],"title":["a paper"],"journaltitle":"unexpected scalars"}]}',
     },
 
+    {
+      label: "all keys (append)",
+      src: {path:"$.results[*]", key:"*"},
+      dst: {append:jsonPathHelper.APPEND, path:"$.results[*]", key:"ignored"},
+      func: function (el) {
+        if (typeof el !== 'object') return el.toUpperCase();
+        else return '##NOOP##';
+      },
+      obj: JSON.parse(record),
+      isArrayMap: false,
+      expected: '{"results":[{"author":["billy","bob","BILLY","BOB"],"title":["a book","A BOOK"],"item":[{"location":["a library"]},{"location":["my house"]}]},{"author":["jill","JILL"],"title":["a paper","A PAPER"],"journaltitle":["unexpected scalars","UNEXPECTED SCALARS"]}]}',
+    },
+
     {
       label: "in place (sparse, replace scalar)",
       src: {path:"$.results[*]", key:"journaltitle"},
@@ -823,7 +844,6 @@ var jsonPathHelper = {
         logger.error(test.expected);
         logger.error("Actual:");
         logger.error(JSON.stringify(test.obj));
-        dump (result);
         return false;
       }
     }
