@@ -19,7 +19,7 @@ var xpatternDebug= false;
 // very often. Maybe only in the builder, or in special cases.
 // Saves some memory...
 function Xpattern() {
-  this.nodeType = "any"; // often a tag like "TD". Or boolean, or special...
+  this.nodeType = "any"; // often a tag like "TD". Or group, or special...
   this.repeatable = false;
   this.optional = false;
   this.greedy = true; // default to true, for historical reasons
@@ -38,6 +38,7 @@ function Xpattern() {
   // this.stringoffset = -1; // offset to the string that defined the pattern
                              // only used in builder to highlight
   // this.modifiers={};
+  // xpatternMaker may alse set .group while working on the stuff. Ignored here.
 } // Xpattern constructor
 
 // All known modifiers
@@ -107,7 +108,10 @@ XpatternHit.prototype.appendString = function (key, txt, domNode, patNode) {
 // breaks with no name are hints to the builder about interesting nodes,
 // so we only pass them on if the builder has asked for nodes to be remembered.
 XpatternHit.prototype.break = function (name, domNode, patNode) {
-  if ( (name == "" || name == "none") & (! patNode.nodesToo) ) {
+  if ( name == "none" ) {
+    return;
+  }
+  if ( name == "" & ! patNode.nodesToo ) {
     return;
   }
   var v = [];
@@ -241,13 +245,16 @@ Xpattern.prototype.recalulateVariable = function() {
   }
 };
 
+// Get an array of all known modifiers (for builder menu)
+Xpattern.getAllModifiers = function() {
+  return xpatternModifiers;  
+}
+
 // Check if the variable has a modifier appended to it
 Xpattern.prototype.varhasmodifier = function(variable,mod) {
   return (variable.indexOf("/"+mod) != -1 );
 };
 
-// TODO - refactor modifiers into their own hash, more proper that way
-
 // Sets a modifier. Returns true if all went well, false in case of
 // a bad modifier.
 Xpattern.prototype.setModifier = function(mod) {
@@ -263,13 +270,27 @@ Xpattern.prototype.setModifier = function(mod) {
   return false; // oops, didn't recognize it
 };
 
+// Get an array of the modifiers (usually only one, but can be more)
+Xpattern.prototype.getModifiers = function() {
+  if ( !this.modifiers ) {
+    return [];
+  }
+  var mods = [];
+  for ( var m in this.modifiers ) {
+    mods.push(m);
+  }
+  return mods;
+};
 
 Xpattern.prototype.setCardinality = function(c) {
-  if ( c == "+" || c == '*' ) {
-    this.repeatable=true;
+  if ( c == "+" || c == "*" || c== "+?" || c == "*?" ) {
+    this.repeatable = true;
   }
-  if ( c == "?" || c == '*' ) {
-    this.optional=true;
+  if ( c == "?" || c == "*" || c == "*?" ) {
+    this.optional = true;
+  }
+  if ( c == "+?" || c == "*?" || c == "??" ) { // Is ?? valid ###
+    this.greedy = false;
   }
 };
 
@@ -451,7 +472,7 @@ Xpattern.prototype.dumpString = function ( indent ) {
     if (this.modifiers && this.modifiers[mod] )
       modifier += " -" + mod ;
   }
-  
+
   var attr="";
   if ( this.attributes != null ) {
     attr = " [ ";
@@ -593,35 +614,82 @@ Xpattern.prototype.findMatch = function ( domNode, hitsArray, siblingstoo ) {
 // returns a XpatternHit with collected hits, if matches (can be emtpy)
 // or null if the match fails.
 // ingroup determines how restmatches are collected
+// Actually, this is mostly concerned about optional nodes, the
+// matching itself happens in matchNonOptional() below
 Xpattern.prototype.matchHere = function ( domNode, varname, ingroup ) {
+  var thishits = this.matchNonOptional( domNode, varname, ingroup );
+  if ( thishits ) {
+    this.trace("matchHere returning hit ", domNode, thishits);
+    return thishits;
+  }
+  // no hit, do we have an alternative
+  if ( this.nextAlternative ) {
+    this.trace("matchHere: mismatch, trying alternative", domNode, null);
+    var althits = this.nextAlternative.matchHere( domNode, varname, ingroup);
+    if ( althits ) {
+      this.trace("matchHere: Found a good alternative, returning that",
+                 domNode, althits);
+      return althits;
+    }
+  }
+  // no hit, check for optional and alternatives
+  if ( this.optional ) {
+    if ( this.nextSibling ) {
+      this.trace("matchHere: Optional node, didn't match, proceeding",
+                 domNode, null);
+      return this.nextSibling.matchHere( domNode, varname, ingroup );
+    }
+    this.trace("matchHere: Optional node didn't match, nothing to follow, " +
+        "returning empty hits (and collecting rests)", domNode, null );
+    if ( !ingroup ) {
+      // but not if inside a group, its hits must come from inside!
+      var resthits =  this.collectRest(domNode, varname);
+      this.trace("matchHere: After collecting rest ", domNode, resthits);
+      return resthits;
+      // Collect also this node itself, as it has failed to match, so
+      // we don't have it in the hits.
+    } else {
+      this.trace("matchHere: not collecting hits, since inside a group",
+          domNode, null);
+      return new XpatternHit();
+    }
+  }
+}; // matchHere
+
+
+// Matches the pattern node against the current domNode.
+// returns a XpatternHit with collected hits, if matches (can be emtpy)
+// or null if the match fails.
+// ingroup determines how restmatches are collected
+Xpattern.prototype.matchNonOptional = function ( domNode, varname, ingroup ) {
   // #comment nodes don't matter at all, skip here
   // (without recursion)! - unless a comment pattern or a group
   while (domNode != null && domNode.nodeName.toUpperCase() == "#COMMENT" &&
          this.nodeType.toUpperCase() != "#COMMENT" &&
          this.nodeType != "(" && this.nodeType != "|" ) {
-    this.trace("matchHere skipping comment", domNode, null);
+    this.trace("matchNonOpt skipping comment", domNode, null);
     domNode = domNode.nextSibling;
   }
   if (domNode == null) { // nothing to match against
     if (this.allOptional() ) {
-      this.trace("matchHere beginning, no node, optional pattern",
+      this.trace("matchNonOpt beginning, no node, optional pattern",
          domNode, null);
       return new XpatternHit();
     } else {
       return null; // mismatch
     }
   }
-  this.trace("matchHere starting (ingroup=" + ingroup + ")", domNode,null);
+  this.trace("matchNonOpt starting (ingroup=" + ingroup + ")", domNode,null);
 
   // Check non-greedy optional
   if ( this.optional && ! this.greedy && this.nextSibling != null ) {
-    this.trace("matchHere: trying non-greedy optional next", domNode, null);
+    this.trace("matchNonOpt: trying non-greedy optional next", domNode, null);
     var nexthits = this.nextSibling.matchHere( domNode, varname, ingroup);
     if (nexthits != null) {
-      this.trace("matchHere: non-greedy optional matched ", domNode, nexthits);
+      this.trace("matchNonOpt: non-greedy optional matched ", domNode, nexthits);
       return nexthits;
     }
-    this.trace("matchHere: non-greedy optional failed to match ", domNode, nexthits);
+    this.trace("matchNonOpt: non-greedy optional failed to match ", domNode, nexthits);
   }
 
   // match the node itself
@@ -660,68 +728,45 @@ Xpattern.prototype.matchHere = function ( domNode, varname, ingroup ) {
       }
 
   } else if (domNode.nodeName == "#text" ) {
-    // FIXME - Second possible place for this  (??)
-      // now we can safely collect the text
-      // a dom #text node matches against any real pattern node
-      return this.collectTextNode( domNode, varname, ingroup, "nongreedy");
+    // Second possible place for this  (??)
+    // now we can safely collect the text
+    // a dom #text node matches against any real pattern node
+    return this.collectTextNode( domNode, varname, ingroup, "nongreedy");
+    // TODO - Is this greedy/nongreedy trickery needed any more, after the
+    // matchNonOptional stuff (CP-3451)? Seems not to harm.
   } else { // regular node
     thishits = this.matchNode(domNode, varname);
     // including matches from attributes and childnodes
   }
 
-  this.trace("matchHere after matchNode", domNode, thishits);
+  this.trace("matchNonOpt after matchNode", domNode, thishits);
 
   // check negation
   if (thishits && this.negation ) {
       var neghits = this.negation.matchHere( domNode, varname, ingroup );
       if (neghits) {
-          this.trace("matchHere: Negation matched, nulling hits",
+          this.trace("matchNonOpt: Negation matched, nulling hits",
                      domNode, neghits);
           thishits = null;
       }
   }
 
   if (thishits == null) { // this node failed to match
-    this.trace("matchHere: matchNode did not match", domNode, thishits);
-    if (!this.optional) {
-      return null; // a definite mismatch
-    } else { // optional pattern node, so we do not call it a mismatch
-      this.trace("matchHere: Mismatch, but it was optional", domNode, null);
-      if (this.nextSibling != null) {
-        return this.nextSibling.matchHere(domNode, varname, ingroup);
-        // just return the hits from the nextSibling,
-        // and forget what we may have of this node
-      } else {
-        this.trace("matchHere: Mismatch (optional) at the end", domNode, null);
-        // There may still be text nodes around, collect them.
-        var resthits=null;
-        if ( !ingroup ) {
-          // but not if inside a group, its hits must come from inside!
-          resthits = this.collectRest(domNode, varname);
-          // Collect also this node itself, as it has failed to match, so
-          // we don't have it in the hits. 
-        } else {
-          this.trace("matchHere: not collecting hits, since inside a group",
-              domNode, null);
-        }
-        if ( resthits == null ) {
-          resthits = new XpatternHit(); // it is ok, even with no hits
-        }
-        return resthits;
-      }
-    } // optional
+    this.trace("matchNonOpt: matchNode did not match", domNode, thishits);
+    return null; // declare a mismatch, optional stuff is handled higher up
+                 // in matchHere()
   } else {  // This node matches
     thishits.lastDomNode = domNode; // Remember end marker
-    this.trace("matchHere: matchNode matched", domNode, thishits);
+    this.trace("matchNonOpt: matchNode matched", domNode, thishits);
     if (domNode.nextSibling == null) { // dom tree ends here
         if (this.nextSibling != null) {
             // oops, pattern continues, but nothing to match
             if (this.nextSibling.allOptional()) {
-              this.trace("matchHere: Mismatch, but optional all the way. "+
+              this.trace("matchNonOpt: Mismatch, but optional all the way. "+
                          "Call it a hit.", domNode,thishits);
                 return thishits; // luckily it was all optional nodes
             }
-            this.trace("matchHere:  Mismatch, pattern continues, " +
+            this.trace("matchNonOpt:  Mismatch, pattern continues, " +
                        "but nothing to match", domNode,null);
             return null; // must be a mismatch!
         } else { // both dom and pattern end. That matches.
@@ -729,20 +774,20 @@ Xpattern.prototype.matchHere = function ( domNode, varname, ingroup ) {
         }
     } else { // Dom tree continues
       if (this.repeatable && this.greedy) { // greedy repeat
-        this.trace("matchHere: checking (greedy) repeat",domNode,thishits);
-        var rephits = this.matchHere(domNode.nextSibling, varname, ingroup );
-          if (rephits != null) {
-            this.trace("matchHere: Got repeated hits",domNode,rephits);
-            thishits.appendHits(rephits,true);
-            return thishits;
-          }
+        this.trace("matchNonOpt: checking (greedy) repeat",domNode,thishits);
+        var rephits = this.matchNonOptional(domNode.nextSibling, varname, ingroup );
+        if (rephits != null) {
+          this.trace("matchNonOpt: Got repeated hits",domNode,rephits);
+          thishits.appendHits(rephits,true);
+          return thishits;
+        }
       }
       if (this.nextSibling == null)
       {
           // end of pattern, now we have a real match
         thishits.lastDomNode = domNode; // remember how far we got
-        this.trace("matchHere: End of pattern chain. Setting last. " +
-                    "Collecting rest into $" + varname + " " +
+        this.trace("matchNonOpt: End of pattern chain. Setting last. " +
+                    "Collecting rest into '$" + varname + "' " +
                     "ingroup= " + ingroup, domNode, thishits);
         var resthits; // collect the rest of the hits
         if ( !ingroup ) {
@@ -751,7 +796,7 @@ Xpattern.prototype.matchHere = function ( domNode, varname, ingroup ) {
           thishits.appendHits(resthits,false);
           // no need to set lastDomNode here, I suppose. Hence the 'false'.
         } // the group has collected its stuff already
-        this.trace("matchHere: End of pattern chain ", domNode, thishits );
+        this.trace("matchNonOpt: End of pattern chain ", domNode, thishits );
         return thishits;
       } else { // check the rest of the pattern
         var nexthits = this.nextSibling.matchHere( domNode.nextSibling,
@@ -759,19 +804,19 @@ Xpattern.prototype.matchHere = function ( domNode, varname, ingroup ) {
         if (nexthits == null) {
           if (this.repeatable && !this.greedy ) {
             // try a non-greedy repeat only after regular childnode failed
-            this.trace("matchHere: nongreedyrepeat check", domNode, null );
-            nexthits = this.matchHere( domNode.nextSibling, varname, ingroup);
-            //this.trace("matchHere: nongreedyrepeat: ", domNode, nexthits );
+            this.trace("matchNonOpt: nongreedyrepeat check", domNode, null );
+            nexthits = this.matchNonOptional( domNode.nextSibling, varname, ingroup);
+            //this.trace("matchNonOpt: nongreedyrepeat: ", domNode, nexthits );
             if ( nexthits ) {
-              this.trace("matchHere: nongreedyrepeat matched: ",
+              this.trace("matchNonOpt: nongreedyrepeat matched: ",
                          domNode, nexthits );
-              //dump("matchHere: DEBUG nongreedyrepeat match. " +
+              //dump("matchNonOpt: DEBUG nongreedyrepeat match. " +
               // "thishits.last = '" + dompath(thishits.lastDomNode) +"' " +
               // "nexthits.last = '" + dompath(nexthits.lastDomNode)+ "'" );
               thishits.appendHits(nexthits,true);
               return thishits;
             } else {
-              this.trace("matchHere: nongreedyrepeat mismatch: ",
+              this.trace("matchNonOpt: nongreedyrepeat mismatch: ",
                          domNode, nexthits );
               return null;
             }
@@ -779,7 +824,7 @@ Xpattern.prototype.matchHere = function ( domNode, varname, ingroup ) {
           if (this.optional) {
                // we have already tried non-greedy optional earlier,
                // so if we are non-greedy, we know nextSibling won't match!
-            this.trace("matchHere: NextSibling mismatch, " +
+            this.trace("matchNonOpt: NextSibling mismatch, " +
                 "but this was optional",
                 domNode,null);
             return this.nextSibling.matchHere(domNode,
@@ -788,11 +833,11 @@ Xpattern.prototype.matchHere = function ( domNode, varname, ingroup ) {
             // empty list on optional nodes anyway (not null)?
             // If we do get here, what happens to thishits?
           }
-          this.trace("matchHere: nextSibling mismatch", domNode,null);
+          this.trace("matchNonOpt: nextSibling mismatch", domNode,null);
           return null; // didn't match anyway
         } else {
-          this.trace("matchHere: NextSibling match.", domNode,nexthits);
-          //this.trace("matchHere DEBUG: " +
+          this.trace("matchNonOpt: NextSibling match.", domNode,nexthits);
+          //this.trace("matchNonOpt DEBUG: " +
           //    "thishits.last = '" + dompath(thishits.lastDomNode) + "' " +
           //    "nexthits.last = '" + dompath(nexthits.lastDomNode)+ "'",
           //    domNode, thishits);
@@ -805,7 +850,7 @@ Xpattern.prototype.matchHere = function ( domNode, varname, ingroup ) {
     } // dom has nextSibling
   } // this node matches
   // We should never reach here, all branches are supposed to return...
-  this.trace("matchHere: Reached the end?!!!",domNode,thishits);
+  this.trace("matchNonOpt: Reached the end?!!!",domNode,thishits);
   return thishits;
 }; // matchHere
 
@@ -839,32 +884,15 @@ Xpattern.prototype.matchGroup = function (domNode, varname) {
 }; // matchGroup
 
 // Matches an OR group
-// That is, tries each child node in turn, until one matches
-// and returns that as the hit.
+// Trivially match the first child, and let matchHere worry about alternatives.
 Xpattern.prototype.matchOr = function (domNode, varname) {
   this.trace("matchOr ", domNode, null);
   var thishits = new XpatternHit();
   var childNode = this.firstChild;
-  var childhits = null;
-  while ( childhits == null ) { // try each child in turn
-    childNode.trace("matchOr: trying child ", domNode, null);
-    childhits = childNode.matchHere(domNode,varname, true);
-    if ( childhits == null ) {
-      childNode.trace("matchOrChild: child mismatch, tried", childNode, null);
-      if ( childNode.nextAlternative == null) {
-        childNode.trace("matchOrChild: no more children, definetly mismatch",
-            childNode, null);
-        return null;
-      }
-    } else {
-      childNode.trace("matchOr: matched child ", domNode, childhits);
-    }
-    childNode = childNode.nextAlternative;
-  } // try
-  // if we get so far, we have a hit
-  thishits.appendHits(childhits,true);
-  this.trace("matchOr: Returning hit ", domNode, thishits);
-  return thishits;
+  if ( this.firstChild ) 
+    return this.firstChild.matchHere(domNode, varname, true);
+  else
+    return null; 
 }; // matchOr
 
 
@@ -1109,7 +1137,7 @@ Xpattern.prototype.collectRest = function ( domNode, varname ) {
   }
   if (varname == "" ) {  // no need to follow if we don't want the data
     this.trace("collectRest: not bothering, no $variable",domNode,null);
-    return null;
+    return new XpatternHit(); // return success, albeit empty
   }
   this.trace("collectRest into $"+varname, domNode, null);
   var thishits = new XpatternHit();
