@@ -19,6 +19,14 @@ Components.utils.import('resource://indexdata/util/xpatternText.js');
 function XpatternTester() {
 }
 
+// A helper so we can write KNOWNFAIL(4) in the tests, instead of -4
+// That makes it much more visible. When working with a known fail test,
+// put -KNOWNFAIL(4) in the test (with a minus in front of it, so we get
+// a positive number, and the test will fail. Or 9999+KNOWNFAIL(4), to
+// guarantee a failure)
+function KNOWNFAIL(x) {
+  return -x;
+}
 
 //// Define tests here
 // This is an array of tests.
@@ -35,11 +43,10 @@ function XpatternTester() {
 //    in. Note that it also counts the empty 'break' hits.
 //
 // If you have a test case that you know will fail, comment it well, and
-// put KNOWNFAILURE in exp2. That way, the failure gets noted in
+// put a negative value in exp2. That way, the failure gets noted in
 // unit tests, and if some future change happens to fix it, you will get
 // notified, and should re-enable the test.
 
-const KNOWNFAILURE=-9999;
 
 var testArray = [
 
@@ -144,6 +151,7 @@ var testArray = [
       "<a href='..'>title 5</a><i>id</i><b>au</b><br/>" +
       "<a href='..'>title 6</a><b>au</b><b>au2</b><i>id</i><i>id2</i><br/>" +
       "<a href='..'>title 7</a><b>au</b><i>id</i><b>au2</b><i>id2</i><br/>" +
+      "<a href='..'>title 8</a><i>id</i><i>id2</i><br/>" +
       "",
     [
       // simple match. Gets title 3 only
@@ -151,15 +159,21 @@ var testArray = [
       // Simple alternative, gets titles 2 and 3
       [ "A $title : ( I $id | B $author ) : BR ", 2, 8 ],
       // Repeated alternative, gets titles 2 to 7
-      [ "A $title : ( I $id | B $author ) + : BR ", 6, 40 ],
+      [ "A $title : ( I $id | B $author ) + : BR ", 7, 46 ],
       // Repeated opt. alternative, gets them all
-      [ "A $title : ( I $id | B $author ) * : BR ", 7, 42 ],
+      [ "A $title : ( I $id | B $author ) * : BR ", 8, 48 ],
       // Repeated opt. alternative, gets them all, even without closing BR
-      [ "A $title : ( I $id | B $author ) * ", 7, 42 ],
-      // Alternative with repeated elements, gets titles 
-      [ "A $title : ( I + $id | B + $author ) : BR ", 2, 8 ],
+      [ "A $title : ( I $id | B $author ) * ", 8, 48 ],
+
+      // Match either I's or B's, until first mismatch
+      [ "A $title : ( I + $id | B + $author )", 7, 32  ],
+
+      // Alternative with repeated elements, gets titles
+      // (because of the BR, returns only those where we get the whole pattern
+      // that is titles 2, 3, and 8 , with only one type of followers.
+      [ "A $title : ( I + $id | B + $author ) : BR ", 3, 14 ], 
       // Repeated or with repeated elements, gets titles
-      [ "A $title : ( I + $id | B + $author ) + : BR ", 6, 40 ],
+      [ "A $title : ( I + $id | B + $author ) + : BR ", 7, 46 ],
     ]
   ], // or
   
@@ -176,9 +190,9 @@ var testArray = [
       // simple match. Gets them all
       [ "SPAN $title : BR ", 6, 12 ],
       // Get all that have a class. That is 2,4,5,6
-      // These FAIL, looks like everything has a class
-      [ "SPAN $title [ @class $class] : BR ", 4, KNOWNFAILURE ],
-      [ "SPAN $title [ @class ] : BR ", 4, KNOWNFAILURE ],
+      // These FAIL, looks like everything has a class, even if empty
+      [ "SPAN $title [ @class $class] : BR ", 4, KNOWNFAIL(22)  ],  
+      [ "SPAN $title [ @class ] : BR ", 4, KNOWNFAIL(8) ],
       
       // Get all with a given value.
       // (the backslash is just to quote the quote inside a quoted string
@@ -196,9 +210,9 @@ var testArray = [
       // Get all that match a class and style
       [ "SPAN $title [ @class =\"fooclass\" , @style=\"foostyle\" ] : BR ", 2, 4 ],
 
-      // Get all that match fooclass and extract style
-      [ "SPAN $title [ @class =\"fooclass\" , @style $style ] : BR ", 2, KNOWNFAILURE ],
-      // TODO - Fails, gets all three with fooclass, with emtpy 'break' if none
+      // Get all that match fooclass and extract style: 
+      [ "SPAN $title [ @class =\"fooclass\" , @style $style ] : BR ", 2, KNOWNFAIL(2) ],
+      // TODO - Fails, gets all three with fooclass, even if no style, as in 2
       // in title2
 
       // TODO See bug 3407 which describes the problems with the failing tests.
@@ -259,8 +273,8 @@ var testArray = [
       [ "B $title : BR : #text $author : #comment ? $comment : #text ? $author :" +
         "I ? $author : #text * $author : U ? $author : #text ? $author : BR ",  3, 27 ],
         
-      // Fails: misses the last 'Author' 
-      [ "B $title : BR : #text * $au :  ANY * $au ! BR : #text * $au : BR ",  3, KNOWNFAILURE ],
+      // Fails: misses the last word 'Author'  in 'very complex author' ###
+      [ "B $title : BR : #text * $au :  ANY * $au ! BR : #text * $au : BR ",  3, KNOWNFAIL(24) ],
 
       // Another apporach
       [ "B $title : BR : ( #text $au | i $au | u $au ) +  : #text ? $au: BR",  3, 25 ],
@@ -287,7 +301,7 @@ var testArray = [
       // (sequence of) comment in the middle. Finds all three hits,
       // but of course gets only the beginning of the third title, as it
       // has no provisions for the I and U nodes.
-      [ "B {  #text $title  : #comment * $comment : #text * $title }", 3,10 ],
+      [ "B {  #text $title  : #comment * $comment : #text * $title }", 3, 10 ], 
       // This was a problem (see bug 3406), but the fix for 4508 seemed
       // to fix this too. If the first text had been with a +, it would have eaten
       // the comment.
@@ -298,16 +312,17 @@ var testArray = [
 
       // The following pattern is a good workaround, using the non-greedy
       // repeat, which tries the comment before more text
-      [ "B {  #text +? $title  : #comment * $comment : #text * $title }", 3,10 ],
-
+      [ "B {  #text +? $title  : #comment * $comment : #text * $title }", 3, 10 ], 
 
       // More flexible way to do the same      
       [ "B $title { ( #comment $comment | #text ) * }", 3, 12 ],
-      [ "B $title { ( #text | #comment $comment ) * }", 3, KNOWNFAILURE ],
+      
+      [ "B $title { ( #text | #comment $comment ) * }", 3, KNOWNFAIL(12) ],
       // TODO - This fails too, misses the comment
       // Something about skipping comments too quickly
       // See Bug 3406. After 4508, fails to find anything because it collects
       // the textnode into $title before it gets to the or-group at all.
+      // After CP-3451 gets the title again, but still misses the comment
       // So, we need to put the #comment first, if in a group. Pretty unlikely case!
     ]
   ], // textnode
@@ -375,6 +390,56 @@ var testArray = [
       [ "TD { A $title: (#text $publisher [/Pub/] | #text $author [/Wri/] | BR )+}", 2,12 ],
     ]
   ], // BUG-CP-3272-textnode-2
+  
+  [ "BUG-CP-3451-textnode-3" ,
+      // See CP-3451 - text nodes following optional nodes fail to match
+    "<h1>test page</h1>" +
+    "<table>" +
+    "<tr><td><a href='http://'>title 1 </a>"+
+    "First description <br/>" +
+    "</td></tr>" +
+    "<tr><td>"+
+    "Second description<br/>" +
+    "</td></tr> "+
+    "<tr><td><a href='http://'>title 3 </a><a href='http://'>Subtitle-3</a>"+
+    "third description <br/>" +
+    "</td></tr>" +
+    "<tr><td><b>BOLD-4</b><a href='http://'>title 4</a>"+
+    "fourth description <br/>" +
+    "</td></tr>" +
+    "<tr><td><b>BOLD-5</b>"+
+    "fifth description <br/>" +
+    "</td></tr>" +
+    "</table>" +
+    "",
+    [
+      // Gets the titles all right
+      [ "TD { A $title }", 2,4 ],
+      // Gets one title and some text 
+      [ "TD { A $title: #text $description }", 1,4 ],
+      
+      // This is the canonical case that failed 
+      // Finds the first, but misses the second
+      // Misses the rest, but that is as expected
+      [ "TD { A? $title: #text $description }", 2, 6 ], 
+      // This works
+      [ "TD { A+ $title: #text $description }", 2, 10 ],
+      // A simple workaround
+      [ "TD { ( A $title)? : #text $description }", 2, 6 ],
+      // The following used to fail, but works after the unskippedDomNode trick
+      [ "TD { ( A? $title) : #text $description }", 2, 6 ],
+      // Testing with a regular B tag preceding the problem
+      [ "TD { B $author : A $title : #text $description }", 1,6 ],
+      // Seems not to make much of a difference
+      [ "TD { B $author : A? $title : #text $description }", 2, 10],
+      // Test to see the description ending in $outer
+      [ "TD $outer { A $title: #text $description }", 1,5 ],
+      [ "TD $outer { A? $title: #text $description }", 2, 8 ],
+      // Test with more optionals. Did not find anything at all!!
+      [ "TD { B? $bold : A? $title : I? $italics : #text $description }", 4, 16 ],
+
+    ]
+  ], // BUG-CP-3451-textnode-3
 
   [ "negation-1" ,
     "<h1>test page</h1>" +
@@ -463,14 +528,19 @@ var testArray = [
       [ "DIV  $text { B $bold | I $italic } ", 5, 23 ],
 
       // This gets all hits, even the fifth, after the allOptional
-      // fix around 4508. Of course, nothing can ever get to $italic,
-      // since if B fails, it gets accepted anyway
-      [ "DIV  $text { B ? $bold | I $italic } ", 6, 23 ],
+      // fix around 4508.
+      //[ "DIV  $text { B ? $bold | I $italic } ", 6, 23 ],
+      [ "DIV  $text { B ? $bold | I $italic } ", 6, 25 ],
+      // This changed behavior around CP-3451, now the | takes precedence,
+      // so we try to match the I as well, and we get the i2.   ###
+      // Used to be ,23.
+
 
       // This matches all hits. Again, the second line goes only to $text,
       // but that is actually OK, same as above, the B? misses the I, but
       // being optional, gets accepted with no hits.
-      [ "DIV  $text { B ? $bold | I ? $italic } ", 6, 23 ],
+      [ "DIV  $text { B ? $bold | I ? $italic } ", 6, 25 ],
+      // CP-3451: was 23 hits, because the I 
 
       // This is a better way to write optional alternative stuff
       [ "DIV  $text { ( B $bold | A $url ) ? } ", 6, 23 ],
@@ -636,11 +706,12 @@ var testArray = [
         " #text $date [ /Date/ ]: P " , 3, 14 ],
 
       // Trying to explicitly avoid getting the author in the title
-      //[ "HR: #text ? $title ! #text [ /By/ ]: BR ? : #text ? $author [ /By/ ] : "+
-      //  " BR ? : #text ? $date [ /Date/ ] : P  " , 5, -16 ],
-      // TODO - This fails, gives an empty hit for the fourth. Bug 4554
+      [ "HR: #text ? $title ! #text [ /By/ ]: BR ? : #text ? $author [ /By/ ] : "+
+        " BR ? : #text ? $date [ /Date/ ] : P  " , 5, 14 ],
+      // This used to fail, gives an empty hit for the fourth. Bug 4554
       // (There was a long side track of failed experiments. The problem was found
       // to be in the BRs, which ate the text in a bit too greedy way. See bug 4508.
+      // Fixed in the matchUnconditional fix in CP-3451
         
       // Finally, this gets them all. All BRs must be optional, since they can be missing
       // in various places (fifth and third)
@@ -655,8 +726,26 @@ var testArray = [
                "#text $title | BR ) + : P", 5, 20 ],
     ]
   ], // textmatch
-
-  // Similar to above, but with #comment nodes
+  // Bug CP-3396: Text mismatch
+  // Original pattern in the bug was:
+  //    tr { td : td $author { ( #text ? $callno [ /Call / ] ) $item } } :
+  [ "Bug-3396-textnode",
+    "<h1>Page title</h1>" +
+    "<table><TR>" +
+    "<TD>nevermind</TD>" +
+    "<TD>Call 17 <b>author</b> </TD>" +
+    "</TR></table>" +
+    "",
+    [
+      // The original problem
+      [ "TR { TD: TD $author { ( #text ? $callno [ /Call / ] ) $item } } ",1, 6 ],
+        // This actually works!
+        // TODO - This test may be removed after the first release in 2013. The
+        // bug has been fixed before it was reported.
+    ]
+  ],
+ 
+  // Similar to textmatch above, but with #comment nodes
   [ "commentmatch",
     "<h1>Page title</h1>" +
     "<hr/><!--First title--><br/><!--By First author--><br/><!--Date: 2001--><p/>" +
@@ -675,12 +764,12 @@ var testArray = [
       // and with date too, optionals in BRs as well
       // Gets all 5 hits, but a title "By fourth author"
       [ "HR: #comment $title : BR ? : #comment ? $author [ /By/ ] : "+
-        " BR ? : #comment ? $date [ /Date/ ]  " , 5, 16 ],
+        " BR ? : #comment ? $date [ /Date/ ]  " , 5, 20 ],  // used to be 16. CP-3451
 
     // Trying to explicitly avoid getting the author in the title
-    //[ "HR: #comment ? $title ! #comment [ /By/ ]: BR ? : #comment ? $author [ /By/ ] : "+
-    //  " BR ? : #comment ? $date [ /Date/ ] : P  " , 5, -16 ],
-    // TODO - This fails, gives an empty hit for the fourth. Bug 4554
+    [ "HR: #comment ? $title ! #comment [ /By/ ]: BR ? : #comment ? $author [ /By/ ] : "+
+      " BR ? : #comment ? $date [ /Date/ ] : P  " , 5, 18 ],
+    // Bug CP-2546 CP-3451 seems to have fixed this.
 
     // Trying to fix the same with an OR-bag
     // Ignoring the BRs in between.
@@ -748,11 +837,29 @@ var testArray = [
       [ "B $title : span $inner -whitespace { ANY+ $inside }", 2, 14 ],
     ]
   ], // textcontent-modifier
+  // CP-1738: $none gets set
+  [ "CP-1738-$none",
+    "<h1>test page</h1>" +
+      "<i>first</i><span>Some text</span>" +
+      "<i>second</i><span>More <b>bold</b> text</span>" +
+      "",
+    [
+      [ "i $title : span $description", 2, 10],
+      [ "i $title :span $none", 2, 4],
+      [ "i $title :span $description { B $none }", 1, 5],
+      [ "i $title :span $description { B ? $none }", 2, 9],
+      [ "i $title :span $description { B $none [ /bold/ ] }", 1, 5],
+      [ "i $title :span $description { B $none [ /nothing/ ] }", 0, 0],
+      [ "i $title :span $description { B ? $none [ /bold/ ] }", 2, 9],
+      [ "i $title :span $description { B ? $none [ /nothing/ ] }", 2, 10],
+    ]
+  ], // textcontent-modifier
 
 ]; // testArray
 
 var tests = 0;
 var sets = 0;
+var knownfails = 0;
 
 // Little helper to remove all whitespace
 XpatternTester.prototype.cleanstr = function ( str ) {
@@ -780,12 +887,12 @@ XpatternTester.prototype.onePattern = function ( title, doc, patt, exp1, exp2 )
     for ( var i in hitsarr ) {
         hitelems += hitsarr[i].hits.length;
     }
-    if ( (!failmsg) && hitsarr.length != exp1 ) {
+    if ( (!failmsg) && hitsarr.length != Math.abs(exp1) ) {
       failmsg = "Test " + title + " FAILED! \n" ;
       failmsg += "Expected " + exp1 + " hits, got " +hitsarr.length +
         " (and " + hitelems + "/" + exp2 + " elements)\n";
     }
-    if ( (!failmsg) && hitelems != exp2 ) {
+    if ( (!failmsg) && hitelems != Math.abs(exp2) ) {
       failmsg = "Test " + title + " FAILED! \n" ;
       failmsg += "Got " + exp1 + " hits all right, but " +
         hitelems + " hit elements instead of " + exp2 + "\n";
@@ -797,11 +904,17 @@ XpatternTester.prototype.onePattern = function ( title, doc, patt, exp1, exp2 )
         failmsg += "  in " + e.fileName + "  line " + e.lineNumber + "\n";
     }
   }
+  if ( failmsg && ( exp1 < 0 || exp2 < 0 ) ) {
+    dump("Test " + title + " is a known failure \n");
+    knownfails++;
+    return true;
+  }
+  if ( exp1 < 0 || exp2 < 0 ) {
+    failmsg = "Test " + title + " was a known failure, but IT WORKED\n";
+    // Yes, this is a serious failure. Need to investigate why a test
+    // suddenly starts working, and if that is true, fix the test!
+  }
   if ( failmsg ) {
-    if ( exp2 == KNOWNFAILURE ) {
-      dump("Test " + title + " is a known failure \n");
-      return true;
-    }
     if ( hitsarr ) {
       dump (failmsg);
       for ( var i in hitsarr ) {
@@ -865,8 +978,13 @@ XpatternTester.prototype.unitTest = function ( ) {
     if ( !this.oneTest(testArray[t]) )
       return false;
   }
-  dump("XpatternTester: All " + tests + " tests in " + sets +
-       " sets passed ok\n");
+  if ( knownfails )
+    dump("XpatternTester: " + (tests-knownfails) + " tests of " + tests +
+       " passed (in " + sets + " sets), and " + knownfails + " were known fails. " +
+       "That is OK.\n");
+  else 
+    dump("XpatternTester: All " + tests + " tests in " + sets +
+       " sets passed OK!\n");
   return true;
 };
 
