@@ -12,6 +12,9 @@ Components.utils.import('resource://indexdata/thirdparty/jsonPath.js');
 Components.utils.import('resource://indexdata/util/inspector.js');
 Components.utils.import('resource://indexdata/ui/XpathRefine.js');
 Components.utils.import('resource://indexdata/ui/app.js');
+Components.utils.import('resource://indexdata/util/logging.js');
+
+var logger = logging.getLogger();
 
 xulHelper = {
 
@@ -40,7 +43,6 @@ xulHelper = {
       attributes = {};
     attributes["label"] = caption;
     var capt = xmlHelper.appendNode(parentnode, "caption", null, attributes);
-    // */
     return capt;
   },
 
@@ -77,8 +79,8 @@ xulHelper = {
         input.addEventListener("input", function(e) {
             if (confindex != "") {
                 step.conf[confindex] = input.value;
-                //dump("Change event on input [" + confindex  + "] " +
-                //  "'" + input.value + "'\n");
+                logger.debug("Change event on input [" + confindex  + "] " +
+                  "'" + input.value + "'\n");
             }
         }, false);
 
@@ -129,7 +131,7 @@ xulHelper = {
       }
       // Some times the selected item doesn't stick! This forces it
       if ( menulist.selectedIndex < 0 ) {
-          //dump("forcing selectField selection to " + selidx + "\n");
+          logger.debug("forcing selectField selection to " + selidx + "\n");
           menulist.selectedIndex = selidx;
       }
       menulist.addEventListener("command", function(e) {
@@ -152,22 +154,22 @@ xulHelper = {
     prependHistory();
 
     menulist.addEventListener("command", function(e) {
-      //dump("historySelectField: command event v='" + menulist.value + "' " +
-      //       " l='" + menulist.label + "'\n");
+      logger.debug("historySelectField: command event v='" + menulist.value + "' " +
+             " l='" + menulist.label + "'\n");
       step.conf[confindex] = menulist.value;
       prependHistory();
     },false);
 
     menulist.addEventListener("change", function(e) {
-      //dump("historySelectField: change event  v='" + menulist.value + "' " +
-      //    " l='" + menulist.label + "'\n");
+      logger.debug("historySelectField: change event  v='" + menulist.value + "' " +
+          " l='" + menulist.label + "'\n");
       step.conf[confindex] = menulist.value;
       prependHistory();
     },false);
 
     menulist.setNewValue = function(val) {
-      //dump("setNewValue: old='" + step.conf[confindex] + "' " +
-      //     "new='" + val + "'\n");
+      logger.debug("setNewValue: old='" + step.conf[confindex] + "' " +
+           "new='" + val + "'\n");
       menulist.value = val;
       prependHistory(val);
     }; // setnewValue
@@ -176,32 +178,32 @@ xulHelper = {
     function prependHistory(newval) {  // inside historySelectField
       var newhist = [];
       if (newval && newval != step.conf[confindex]) { // new is usually undefined
-        //dump("prependHistory: starting with " +
-        //    "Old: '" + step.conf[confindex] + "' " +
-        //    "new: '" + newval + "' \n");
+        logger.debug("prependHistory: starting with " +
+            "Old: '" + step.conf[confindex] + "' " +
+            "new: '" + newval + "' \n");
         var oldval = step.conf[confindex];
         newhist.push(newval);
         step.conf[confindex] = newval;
         newhist.push(oldval);
       } else {
         newhist.push ( step.conf[confindex] );
-        //dump("prependHistory: starting with '" + step.conf[confindex] + "' \n");
+        logger.debug("prependHistory: starting with '" + step.conf[confindex] + "' \n");
       }
       if ( step.conf[histindex] == undefined){
-        //dump("prependHistory: Creating new history\n");
+        logger.debug("prependHistory: Creating new history\n");
         step.conf[histindex] = [];
       }
-      //dump("prependHistory: before: h.length=" + step.conf[histindex].length + "\n");
+      logger.debug("prependHistory: before: h.length=" + step.conf[histindex].length + "\n");
       var i;
       for ( i=0; (i<step.conf[histindex].length) && (newhist.length<10); i++) {
         if (step.conf[histindex][i] != step.conf[confindex] ) {
-          //dump("_prependHistory: adding '" + step.conf[histindex][i] + "' \n");
+          logger.debug("_prependHistory: adding '" + step.conf[histindex][i] + "' \n");
           newhist.push(step.conf[histindex][i]);
         }
       }
       step.conf[histindex] = newhist;
-      //dump("prependHistory: after: h.length=" + step.conf[histindex].length + "\n");
-      //dump("prependHistory: menupop= " + menupop + "\n");
+      logger.debug("prependHistory: after: h.length=" + step.conf[histindex].length + "\n");
+      logger.debug("prependHistory: menupop= " + menupop + "\n");
       xmlHelper.emptyChildren(menupop);
       for ( i=0; i<step.conf[histindex].length; i++) {
         var val = step.conf[histindex][i];
@@ -225,18 +227,18 @@ xulHelper = {
   // ...or takes a hash with option values mapping to display names,
   // for example { 'ti': 'Title', 'au': 'Author' }
   arraySelectField: function(parentnode, step, confindex, options, extraoptions) {
-    //dump("arraySelectField: Starting with " + options.length + " items -----\n");
+    logger.debug("arraySelectField: Starting with " + options.length + " items -----\n");
     var theSelectedItem = undefined;
     var i;
 
     if ( ! options ) {
       // defensive coding, any kind of non-value will do
-      /*
-      dump("arraySelectField called without options\n");
-      dump("confindex='" + confindex + "'\n");
-      dump("options='" + options + "'\n");
-      dump("exraoptions='" + extraoptions + "'\n");
-      */
+      
+      logger.debug("arraySelectField called without options\n");
+      logger.debug("confindex='" + confindex + "'\n");
+      logger.debug("options='" + options + "'\n");
+      logger.debug("extraoptions='" + extraoptions + "'\n");
+      
       options = [];
     } else if (typeof(options) != "object") {
       throw new Error("Invalid options parameter to arraySelectField.");
@@ -307,11 +309,11 @@ xulHelper = {
 
     if (menulist.selectedItem && menulist.selectedItem.value) {
         if (confindex) {
-            //*dump("*** arraySelectField() setting conf[" + confindex + "]\n");
+            logger.debug("*** arraySelectField() setting conf[" + confindex + "]\n");
             step.conf[confindex] = menulist.selectedItem.value;
-            //*dump("*** arraySelectField() has set conf[" + confindex + "]\n");
+            logger.debug("*** arraySelectField() has set conf[" + confindex + "]\n");
         }
-      //dump("setting step.conf[" + confindex + "] to '" + step.conf[confindex] + "'\n");
+      logger.debug("setting step.conf[" + confindex + "] to '" + step.conf[confindex] + "'\n");
     }
 
     menulist.addEventListener("command", function(e) {
@@ -319,7 +321,7 @@ xulHelper = {
             step.conf[confindex] = menulist.selectedItem.value;
         }
       }, false);
-    //*dump("arraySelectField() returning " + menulist + " = <" + menulist.localName + ">\n");
+    logger.debug("arraySelectField() returning " + menulist + " = <" + menulist.localName + ">\n");
     return menulist;
   },
 
@@ -328,14 +330,14 @@ xulHelper = {
   // at the end of the list
   selectMenulistItemWithValue: function(menulist, value, makeit) {
     var i = 0;
-    dump("selectMenulistItemWithValue() searching '" + menulist + "' " +
+    logger.debug("selectMenulistItemWithValue() searching '" + menulist + "' " +
          "(type '" + typeof(menulist) + "' " +
          "with localName '" + menulist.localName + "') " +
          "for '" + value + "', count = " + menulist.itemCount + "\n");
     while (i < menulist.itemCount) {
       var item = menulist.getItemAtIndex(i);
       
-      dump("selectMenulistItemWithValue() comparing '" + item.value + "' " +
+      logger.debug("selectMenulistItemWithValue() comparing '" + item.value + "' " +
            "with '" + value + "' => " + (item.value == value) + "\n");
       
       if (item.value == value) {
@@ -352,16 +354,16 @@ xulHelper = {
           var newitem = menulist.appendItem( value, value )
           menulist.selectedItem = newitem;
         } catch(e) {
-          //dump("Caught an error when appending a menu item " + e + "\n");
+          logger.debug("Caught an error when appending a menu item " + e + "\n");
           for ( var f in menulist )
             if ( typeof(menulist[f]) == "function" ) // kills some time
-              //dump("menulist has function " + f + "\n");
+              logger.debug("menulist has function " + f + "\n");
           try {
             var newitem = menulist.appendItem( value, value )
             menulist.selectedItem = newitem;
-            //dump("menulist.appendItem worked the second time !!!\n");
+            logger.debug("menulist.appendItem worked the second time !!!\n");
           } catch(e) {
-            //dump("Again caught an error when appending a menu item " + e +"\n");
+            logger.debug("Again caught an error when appending a menu item " + e +"\n");
           }
         }
         // Actually, we should not normally get here at all, but we tend to
@@ -385,8 +387,8 @@ xulHelper = {
   // TODO - At some point we should refactor jsonPathField to use this
   singleJsonPathField: function( surface, step, confindex, caption,
                     jsonPrefix,  groups, extraoptions ) {
-    //dump("singleJsonPathField starting g=" + groups +
-    //     " p='" + jsonPrefix + "'\n");
+    logger.debug("singleJsonPathField starting g=" + groups +
+         " p='" + jsonPrefix + "'\n");
     if (!groups)
         groups = false;
     var hb = xmlHelper.appendNode(surface, "hbox", null, {align: "center"});
@@ -396,12 +398,12 @@ xulHelper = {
     var templ = step.task.getTemplate().properties.data;
     var obj = jsonPath(templ, jsonPrefix);
     if ( !obj ) {
-      //dump("singleJsonPathField: '" + jsonPrefix + "' failed, " +
-      //     "trying output.results\n");
+      logger.debug("singleJsonPathField: '" + jsonPrefix + "' failed, " +
+           "trying output.results\n");
       obj = jsonPath(templ, "$.output.results"); // a good guess
     }
     if ( !obj ) {
-      //dump("singleJsonPathField: '" + jsonPrefix + "' failed, plain output\n");
+      logger.debug("singleJsonPathField: '" + jsonPrefix + "' failed, plain output\n");
       obj = jsonPath(templ, "$.output"); // another good guess
     }
     //obj = obj[0][0]; // it seems always to be a double array ??!!
@@ -411,8 +413,8 @@ xulHelper = {
       obj=obj[0];
     if ( Array.isArray(obj) )
       obj=obj[0];
-    //dump("got obj: " + obj + "\n");
-    //dump(JSON.stringify(obj) + "\n");
+    logger.debug("got obj: " + obj + "\n");
+    logger.debug(JSON.stringify(obj) + "\n");
     var list = [];
     for ( var k in obj ) {
         if ( groups ) {
@@ -425,7 +427,7 @@ xulHelper = {
     }
     var ret = xulHelper.arraySelectField(hb, step, confindex, list,
                                          extraoptions);
-    //dump("singleJsonPathField done \n");
+    logger.debug("singleJsonPathField done \n");
     return ret;
   }, // singleJsonPathField
 
@@ -787,13 +789,33 @@ xulHelper = {
   // tend to be cyclical!)
   highlightNode: function(node, color, userhighlight ) {
     var nodes = [ node, node.cf_displaynode, node.cf_datanode  ];
+    var displaycolor = color;
+    //if ( node.getAttribute && node.getAttribute("cf_user_highlight") == color)
+    //  userhighlight = true; // use has highlighted this before
+    // Doesn't work, userhighlights everything
+    if ( !userhighlight ) { // Dim the color a bit, if not user-selected
+      var m = color.match( /#(..)(..)(..)$/ );
+      if ( m ) {
+        var r = parseInt(m[1],16);
+        var g = parseInt(m[2],16);
+        var b = parseInt(m[3],16);
+        r = Math.floor( r / 2 ) + 127;
+        g = Math.floor( g / 2 ) + 127;
+        b = Math.floor( b / 2 ) + 127;
+        var rgb = r * 256 * 256 + g * 256 + b;
+        displaycolor =rgb.toString(16);
+        while ( displaycolor.length < 6 )
+          displaycolor = "0" + displaycolor;
+        displaycolor = "#" + displaycolor;
+      }
+    }
     for ( var i in nodes ) {
       var n = nodes[i];
       if (n) {
         var old = "" ;
         if ( n.style ) {
           old = n.style.backgroundColor;
-          n.style.backgroundColor=color;
+          n.style.backgroundColor=displaycolor;
         }
         if ( n.getAttribute ) {
           // for some reason, CDATA nodes seem not to have getAttribute()
@@ -801,6 +823,7 @@ xulHelper = {
           if (oldattr == undefined ) {
             n.setAttribute("cf_old_style",old);
           }
+          n.setAttribute("cf_orig_highlight", color); 
           if (userhighlight) {
             n.setAttribute("cf_user_highlight",color);
           }
@@ -809,6 +832,27 @@ xulHelper = {
     }
   }, // highlightNode
 
+  getHighlight: function( n ) {
+    if (!n) //defensive coding, should not happen
+      return "";
+    if ( n.getAttribute ) {
+      if ( n.hasAttribute("cf_orig_highlight") )
+        return n.getAttribute("cf_orig_highlight");
+      if ( n.hasAttribute("cf_user_highlight") )
+        return n.getAttribute("cf_user_highlight");
+    }
+    if ( n.style )
+      return n.style.backgroundColor;
+    return "";
+  }, // getHighlight
+
+  // Reapply the original highlight
+  // in case the node has been highlighted dimly, this brightens it up
+  rehighlightNode: function( n ) {
+      if ( n && n.getAttribute && n.style )
+        n.style.backgroundColor = n.getAttribute("cf_orig_highlight");
+  }, // rehighlightNode
+  
   unhighlightNode: function(node, userhighlight ) {
     var nodes = [ node, node.cf_displaynode, node.cf_datanode  ];
     for ( var i in nodes ) {
@@ -821,6 +865,7 @@ xulHelper = {
               n.style.backgroundColor = old;
             n.cf_old_style = undefined;
           }
+          n.removeAttribute("cf_orig_highlight");
           if (userhighlight) {
             n.removeAttribute("cf_user_highlight");
           }
@@ -844,21 +889,27 @@ xulHelper = {
    * each of the strings in the array of titles, and using the
    * specified set of attributes for the top-level tabbox element.
    * Returns an array of nodes representing the tab panels.
+   * Each tab panel has a member cf_tab that is the tab linked to this
+   * panel, so you can listen on its command event to detect when
+   * the user clicks.
    */
   tabBox: function(node, titles, attributes) {
     var res = [];
     var box = xmlHelper.appendNode(node, "tabbox", null, attributes);
     var tabs = xmlHelper.appendNode(box, "tabs");
+    var tabarray = [];
     for (var i = 0; i < titles.length; i++) {
       var tabattrs = { label: titles[i] };
       if (i == 0) tabattrs.selected = true;
-      xmlHelper.appendNode(tabs, "tab", null, tabattrs);
+      tabarray[i] = xmlHelper.appendNode(tabs, "tab", null, tabattrs);
     }
     var panels = xmlHelper.appendNode(box, "tabpanels", null,
                      { flex: 1 } );
     for (var i = 0; i < titles.length; i++) {
-      res.push(xmlHelper.appendNode(panels, "tabpanel", null, 
-                                    { style:'overflow: auto;'} ));
+      var thistab = xmlHelper.appendNode(panels, "tabpanel", null,
+                  { style:'overflow: auto;'} );
+      thistab.cf_tab = tabarray[i];
+      res.push(thistab);
     }
     return res;
   },
@@ -885,6 +936,47 @@ xulHelper = {
 
     return cb;
   },
+
+  makeValueListener: function(obj, key) {
+    return function(e) {
+      obj[key] = e.target.value;
+    };
+  },
+
+  makeToggleListener: function(obj, key) {
+    return function(e) {
+      obj[key] = !obj[key];
+    };
+  },
+
+  // Builds a menu and a menupopup. not connected to any conf variables
+  // Returns the menupopup, into which you can then add other menus, or
+  // menuitems
+  menu: function (surface, label, attrs) {
+    if (!attrs )
+        attrs = {};
+    if (label )
+        attrs.label = label;    
+    var menu = xmlHelper.appendNode(surface, "menu", null, attrs );
+                                        //{ label:label } );
+    var pop = xmlHelper.appendNode(menu, "menupopup", null, null );
+    return pop;
+  },
+
+  // Adds a menu item into the menu
+  menuitem: function( menu, label) {
+    var item = xmlHelper.appendNode(menu, "menuitem", null,
+                                           { label: label } );
+    return item;
+  },
+
+  // Adds a menu separator into the menu
+  menuseparator: function( menu, label) {
+    var item = xmlHelper.appendNode(menu, "menuseparator", null,
+                                           { label: label } );
+    return item;
+  },
+
 } // xulHelper
 
 } /* END BUILDER ONLY WRAPPER */
