@@ -1,5 +1,5 @@
 /* This file is part of the Connector Framework
- * Copyright (C) 2008-2012 Index Data
+ * Copyright (C) 2008-2013 Index Data
  * See the file LICENSE for details.
  */
 /**
@@ -127,7 +127,7 @@ add_page (GtkNotebook *notebook)
     g_signal_connect(close_button, "clicked", G_CALLBACK(close_cb), view);
 
     moz_web_view_load_data(MOZ_WEB_VIEW(view), "file:///page.html",
-                           "text/html", "", 1);
+                           "text/html; charset=utf-8", "", 0);
     int page = gtk_notebook_append_page(notebook, view, box);
     gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), page);
 
@@ -458,10 +458,12 @@ CF_Engine::CF_Engine(enum Flags flags,
                      const char *builder_path,
                      const char *app_path,
                      const char *profile_path, const char *proxy,
+                     const char *loglevel,
                      CF_Logger *logger, int fd)
 {
     m_priv = 0;
-    init(flags, 0, 0, builder_path, app_path, profile_path, proxy, logger, fd);
+    init(flags, 0, 0, builder_path, app_path, profile_path, proxy,
+         loglevel, logger, fd);
 }
 
 bool CF_Embed::wait_init_handshake(std::string &error_msg)
@@ -566,12 +568,14 @@ bool CF_Embed::spawn_thread(std::string &err_msg)
 void CF_Engine::init(enum Flags flags,
                      int argc, char **argv, const char *builder_path,
                      const char *app_path, const char *profile_path,
-                     const char *proxy, CF_Logger *logger, int fd)
+                     const char *proxy, const char *loglevel,
+                     CF_Logger *logger, int fd)
 {
     if (!builder_path)
 	throw CF_Engine_Error_System("builder_path NULL");
-    CF_Embed *embed = new CF_Embed(argc, argv, builder_path, app_path, profile_path,
-                                   proxy, logger,
+    CF_Embed *embed = new CF_Embed(argc, argv,
+                                   builder_path, app_path, profile_path,
+                                   proxy, loglevel, logger,
                                    fd);
 
     embed->logger->printf("engine",
@@ -643,7 +647,8 @@ void CF_Embed::printf(const char *level, const char *fmt, ...)
 
 
 void CF_Embed::copy_initial_js_files(const char *a_profile_path,
-                                const char *a_app_path)
+                                     const char *a_app_path,
+                                     const char *loglevel)
 {
     WRBUF src_fname = wrbuf_alloc();
     WRBUF dst_fname = wrbuf_alloc();
@@ -676,6 +681,10 @@ void CF_Embed::copy_initial_js_files(const char *a_profile_path,
                     if (rd < sizeof(buf))
                         break;
                 }
+                if (loglevel && *loglevel)
+                    fprintf(dst_file,
+                            "user_pref(\"indexdata.cf.logging.root\", \"%s\");\n",
+                            loglevel);
                 fclose(dst_file);
             }
             fclose(src_file);
@@ -688,6 +697,7 @@ CF_Embed::CF_Embed(int a_argc, char **a_argv,
                    const char *a_app_path,
                    const char *a_profile_path,
 		   const char *a_proxy,
+                   const char *a_loglevel,
                    CF_Logger *a_logger,
                    int fd)
     : logger(a_logger),
@@ -756,7 +766,7 @@ CF_Embed::CF_Embed(int a_argc, char **a_argv,
     {
         mkdtemp(profile_path);
         profile_temp = 1;
-        copy_initial_js_files(profile_path, a_app_path);
+        copy_initial_js_files(profile_path, a_app_path, a_loglevel);
     }
     current_task = wrbuf_alloc();
 }
