@@ -1,5 +1,5 @@
 /* This file is part of the Connector Framework
- * Copyright (C) 2008-2012 Index Data
+ * Copyright (C) 2008-2013 Index Data
  * See the file LICENSE for details.
  */
 
@@ -180,8 +180,9 @@ static void connector_open(sessionhandle ses, bend_search_rr *rr)
     }
     WRBUF json = wrbuf_alloc();
     const char *proxy = 0;
+    const char *loglevel = 0;
     int r = cf_database_args(
-        sep, rr->stream, &ses->subdatabase, &proxy, json,
+        sep, rr->stream, &ses->subdatabase, &proxy, &loglevel, json,
         ses->db_username ? wrbuf_cstr(ses->db_username) : 0,
         ses->db_password ? wrbuf_cstr(ses->db_password) : 0);
     if (r)
@@ -197,6 +198,7 @@ static void connector_open(sessionhandle ses, bend_search_rr *rr)
         {
             ses->cf_engine = server_handle.cf_factory->create(CF_Engine::FORK,
                                                               proxy,
+                                                              loglevel,
                                                               ses,
                                                               ses->log_fd);
         }
@@ -210,7 +212,7 @@ static void connector_open(sessionhandle ses, bend_search_rr *rr)
         // load connector file and run init
         try
         {
-            ses->cf_engine->load_cf(wrbuf_cstr(cf_file));
+            ses->cf_engine->load_cf(wrbuf_cstr(cf_file), wrbuf_cstr(json));
             const char *outp = 0;
             ses->cf_engine->run_task_opt("init", wrbuf_cstr(json), &outp);
         }
@@ -588,9 +590,7 @@ static bool get_record_xml(sessionhandle ses, int n, WRBUF wrb, int *errcode,
     if (!r)
         return false;
     wrbuf_puts(wrb, "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
-    wrbuf_puts(wrb, "<record>\n");
-    make_xml_al(r, wrb, 0);
-    wrbuf_puts(wrb, "</record>\n");
+    json_to_xml(r, wrb, "record");
     return true;
 }
 
