@@ -1,5 +1,5 @@
 /* This file is part of the Connector Framework
- * Copyright (C) 2008-2012 Index Data
+ * Copyright (C) 2008-2013 Index Data
  * See the file LICENSE for details.
  */
 
@@ -93,8 +93,10 @@ namespace metaproxy_1 {
             CF::Session *cf_session;
             int no_searches;
             bool get_next();
-            struct json_node *get_record(int n, int *errcode);
-            bool get_record_xml(int n, WRBUF wrb, int *errcode);
+            struct json_node *get_detailed_record(int n, int *errcode,
+                struct json_node *record_r);
+            struct json_node *get_record(int n, int *errcode, int details);
+            bool get_record_xml(int n, WRBUF wrb, int *errcode, int details);
 
             ZSession(const Z_IdAuthentication *auth);
             ~ZSession();
@@ -116,9 +118,9 @@ namespace metaproxy_1 {
 
 
             Z_Records *fetch_z3950(
-                CF::ZSession *zsession,
-                mp::odr &odr, Odr_oid *preferredRecordSyntax,
-                Z_ElementSetNames *esn,
+                CF::ZSession *zsession, mp::odr &odr,
+                const Odr_oid *preferredRecordSyntax,
+                const Z_ElementSetNames *esn,
                 int start, int number, int &error_code, std::string &addinfo,
                 int *number_returned, int *next_position);
 
@@ -135,7 +137,9 @@ namespace metaproxy_1 {
                                 const char *suffix_path);
 
             CF_Engine *create_connector(CF_Engine::Flags engine_flag,
-                                        const char *proxy, CF::Session *ses,
+                                        const char *proxy,
+                                        const char *loglevel,
+                                        CF::Session *ses,
                                         std::string &addinfo);
 
             bool load_connector(Package &package, Z_HTTP_Request *req,
@@ -294,7 +298,85 @@ bool yf::CF::ZSession::get_next()
     return false;
 }
 
-struct json_node * yf::CF::ZSession::get_record(int n, int *errcode)
+static int json_append_object(struct json_node *dst, struct json_node *src)
+{
+    if (dst && src &&
+        dst->type == json_node_object && src->type == json_node_object)
+    {
+        struct json_node **np = &dst->u.link[0];
+        while (*np)
+            np = &(*np)->u.link[1];
+        *np = src->u.link[0];
+        src->u.link[0] = 0;
+        json_remove_node(src);
+        return 0;
+    }
+    return -1;
+}
+
+struct json_node *yf::CF::ZSession::get_detailed_record(
+    int n, int *errcode, struct json_node *record_r)
+{
+    mp::wrbuf wrb;
+    struct json_node *detail_x;
+    struct json_node *detail_rec = 0;
+
+    wrbuf_printf(wrb, "details%d", n);
+    detail_x = json_get_object(records, wrbuf_cstr(wrb));
+    if (detail_x)
+        detail_rec = json_get_elem(detail_x, 0);
+    else
+    {
+        wrbuf_rewind(wrb);
+        wrbuf_puts(wrb, "{\"record\":");
+        json_write_wrbuf(record_r, wrb);
+        wrbuf_puts(wrb, "}");
+
+        const char *outp = 0;
+        try
+        {
+            cf_session->engine->run_task_opt("details", wrbuf_cstr(wrb), &outp);
+        }
+        catch (CF_Engine_Error &e)
+        {
+            const char *what = e.what();
+            *errcode = YAZ_BIB1_SYSTEM_ERROR_IN_PRESENTING_RECORDS;
+            yaz_log(YLOG_WARN, "details task failed: %s",
+                    what ? what : "null");
+            record_r = 0;
+        }
+        if (!outp || !strcmp(outp, "{}"))
+            return record_r;
+        struct json_node *n_out = json_parse(outp, 0);
+        struct json_node *n_results = json_detach_object(n_out, "results");
+
+        json_remove_node(n_out);
+
+        if (n_results && n_results->type == json_node_array)
+        {
+            json_parser_t parser = json_parser_create();
+
+            wrbuf_rewind(wrb);
+            wrbuf_printf(wrb, "{\"details%d\":", n);
+            wrbuf_puts(wrb, "%0}");
+
+            json_parser_subst(parser, 0, n_results);
+
+            detail_x = json_parser_parse(parser, wrbuf_cstr(wrb));
+
+            json_parser_destroy(parser);
+
+            if (detail_x)
+                json_append_object(records, detail_x);
+            detail_rec = json_get_elem(n_results, 0);
+        }
+    }
+    return detail_rec;
+}
+
+
+struct json_node *yf::CF::ZSession::get_record(int n, int *errcode,
+                                               int details)
 {
     if (!records)
     {
@@ -330,18 +412,20 @@ struct json_node * yf::CF::ZSession::get_record(int n, int *errcode)
             return 0;
         }
     }
-    return json_get_elem(recs, n - 1);
+    struct json_node *r = json_get_elem(recs, n - 1);
+    if (r && details)
+        return get_detailed_record(n, errcode, r);
+    return r;
 }
 
-bool yf::CF::ZSession::get_record_xml(int n, WRBUF wrb, int *errcode)
+bool yf::CF::ZSession::get_record_xml(int n, WRBUF wrb, int *errcode,
+                                      int details)
 {
-    struct json_node *r = get_record(n, errcode);
+    struct json_node *r = get_record(n, errcode, details);
     if (!r)
         return false;
     wrbuf_puts(wrb, "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
-    wrbuf_puts(wrb, "<record>\n");
-    make_xml_al(r, wrb, 0);
-    wrbuf_puts(wrb, "</record>\n");
+    json_to_xml(r, wrb, "record");
     return true;
 }
 
@@ -468,7 +552,9 @@ void yf::CF::Rep::expire(void)
 }
 
 CF_Engine *yf::CF::Rep::create_connector(CF_Engine::Flags engine_flag,
-                                         const char *proxy, CF::Session *ses,
+                                         const char *proxy,
+                                         const char *loglevel,
+                                         CF::Session *ses,
                                          std::string &addinfo)
 {
     if (cf_factory->get_cf_tmp_dir(ses->file_area))
@@ -495,7 +581,8 @@ CF_Engine *yf::CF::Rep::create_connector(CF_Engine::Flags engine_flag,
 
     CF_Engine *cf = 0;
     try {
-        cf = cf_factory->create(engine_flag, proxy, ses, fileno(log_file));
+        cf = cf_factory->create(engine_flag, proxy, loglevel,
+                                ses, fileno(log_file));
     }
     catch (CF_Engine_Error &e)
     {
@@ -565,6 +652,7 @@ void yf::CF::Rep::post_connector(Package &package,
         no_parms = yaz_uri_to_array(args_str, odr, &names, &values);
 
     const char *proxy = 0;
+    const char *loglevel = 0;
     // POST /connector[?proxy=IP]
     bool use_xml = false;
     if (strcmp(req->method, "POST"))
@@ -581,6 +669,8 @@ void yf::CF::Rep::post_connector(Package &package,
     {
         if (!strcmp(names[i], "proxy"))
             proxy = values[i];
+        else if (!strcmp(names[i], "loglevel"))
+            loglevel = values[i];
         else if (!strcmp(names[i], "thread"))
         {
             if (values[i] && *values[i] == '1')
@@ -608,7 +698,8 @@ void yf::CF::Rep::post_connector(Package &package,
     }
 
     std::string addinfo;
-    CF_Engine *cf = create_connector(engine_flag, proxy, ses, addinfo);
+    CF_Engine *cf = create_connector(engine_flag, proxy, loglevel,
+                                     ses, addinfo);
     if (!cf)
     {
         Z_GDU *zgdu_res
@@ -656,6 +747,16 @@ void yf::CF::Rep::post_connector(Package &package,
     ses->in_use = false;
 }
 
+static bool is_xml_content(const char *header)
+{
+    if (!header)
+        return false;
+    const char *cp = strchr(header, '/');
+    if (cp && !strcmp(cp+1, "xml"))
+        return true;
+    return false;
+}
+
 /** \brief operate on existing connector */
 void yf::CF::Rep::task_connector(Package &package, Z_HTTP_Request *req,
                                  CF::Session *cf_session,
@@ -664,6 +765,8 @@ void yf::CF::Rep::task_connector(Package &package, Z_HTTP_Request *req,
     mp::odr odr;
     const char *ctype = z_HTTP_header_lookup(req->headers,
                                              "Content-Type");
+    bool accept_xml = is_xml_content(z_HTTP_header_lookup(req->headers,
+                                                          "Accept"));
     if (!strcmp(args, "load_cf"))
     {
         if (!load_connector(package, req, cf_session))
@@ -760,12 +863,38 @@ void yf::CF::Rep::task_connector(Package &package, Z_HTTP_Request *req,
             = odr.create_HTTP_Response(package.session(), req, 200);
         Z_HTTP_Response *response = zgdu_res->u.HTTP_Response;
 
-        z_HTTP_header_add_content_type(odr, &response->headers,
-                                       "application/json",
-                                       "utf-8");
-        response->content_buf = (char *) output_json;
-        response->content_len = strlen(output_json);
+        const char *errmsg = 0;
+        struct json_node *n = json_parse(output_json, &errmsg);
+        if (!n)
+        {
+            Z_GDU *zgdu_res
+                = odr.create_HTTP_Response(package.session(), req, 200);
+            cf_http_msg(
+                zgdu_res, odr,
+                    errmsg ? errmsg : "invalid JSON returned from engine",
+                400);
+            package.response() = zgdu_res;
+            return;
+        }
+        WRBUF wrb = wrbuf_alloc();
+        if (accept_xml)
+        {
+            z_HTTP_header_add_content_type(odr, &response->headers,
+                                           "text/xml", "utf-8");
+            json_to_xml(n, wrb, "opt");
+            response->content_buf = (char *) wrbuf_cstr(wrb);
+            response->content_len = wrbuf_len(wrb);
+        }
+        else
+        {
+            z_HTTP_header_add_content_type(odr, &response->headers,
+                                           "application/json", "utf-8");
+            response->content_buf = (char *) output_json;
+            response->content_len = strlen(output_json);
+        }
         package.response() = zgdu_res;
+        wrbuf_destroy(wrb);
+        json_remove_node(n);
     }
     else
     {
@@ -864,13 +993,14 @@ Z_APDU *yf::CF::Rep::open_cf_z3950(Z_APDU *req,  CF::ZSession *zsession,
     else
         wrbuf_write(cf_db, zsession->database, db_sep - zsession->database);
     int http_status = 500;
-    WRBUF cf_file = cf_factory->get_cf_file(zsession->username,
-                                            zsession->password,
-                                            wrbuf_cstr(cf_db),
-                                            &http_status);
+
+    WRBUF cf_file_tmp = cf_factory->get_cf_file(zsession->username,
+                                                zsession->password,
+                                                wrbuf_cstr(cf_db),
+                                                &http_status);
     wrbuf_destroy(cf_db);
 
-    if (!cf_file)
+    if (!cf_file_tmp)
     {
         int errcode = 0;
         switch (http_status)
@@ -886,19 +1016,22 @@ Z_APDU *yf::CF::Rep::open_cf_z3950(Z_APDU *req,  CF::ZSession *zsession,
         }
         return odr.create_searchResponse(req, errcode, sr->databaseNames[0]);
     }
-    WRBUF json = wrbuf_alloc();
+    std::string cf_file(wrbuf_cstr(cf_file_tmp));
+    wrbuf_destroy(cf_file_tmp);
+
+    mp::wrbuf json;
     const char *proxy = 0;
+    const char *loglevel = 0;
     int r = cf_database_args(
-        db_sep, odr, &zsession->subdatabase, &proxy, json, 0, 0);
+        db_sep, odr, &zsession->subdatabase, &proxy, &loglevel, json, 0, 0);
     if (r)
     {
-        char *addinfo = (char *) odr_malloc(odr, 40 + wrbuf_len(json));
+        char *addinfo = (char *) odr_malloc(odr, 40 + json.len());
         sprintf(addinfo, "Bad database argument: %s", wrbuf_cstr(json));
 
         Z_APDU *a = odr.create_searchResponse(req,
                                               YAZ_BIB1_TEMPORARY_SYSTEM_ERROR,
                                               addinfo);
-        wrbuf_destroy(json);
         return a;
     }
 
@@ -914,11 +1047,11 @@ Z_APDU *yf::CF::Rep::open_cf_z3950(Z_APDU *req,  CF::ZSession *zsession,
     }
 
     std::string addinfo;
-    CF_Engine *cf = create_connector(CF_Engine::FORK, proxy, ses, addinfo);
+    CF_Engine *cf = create_connector(CF_Engine::FORK,
+                                     proxy, loglevel, ses, addinfo);
     if (!cf)
     {
         delete ses;
-        wrbuf_destroy(json);
         return odr.create_searchResponse(req,
                                          YAZ_BIB1_TEMPORARY_SYSTEM_ERROR,
                                          addinfo.c_str());
@@ -928,8 +1061,8 @@ Z_APDU *yf::CF::Rep::open_cf_z3950(Z_APDU *req,  CF::ZSession *zsession,
     // load connector file and run init
     try
     {
-        zsession->cf_session->engine->load_cf(wrbuf_cstr(cf_file));
-
+        zsession->cf_session->engine->load_cf(cf_file.c_str(),
+                                              wrbuf_cstr(json));
         const char *outp = 0;
         zsession->cf_session->engine->run_task_opt("init",
                                                    wrbuf_cstr(json), &outp);
@@ -937,10 +1070,6 @@ Z_APDU *yf::CF::Rep::open_cf_z3950(Z_APDU *req,  CF::ZSession *zsession,
     catch (CF_Engine_Error_System &e)
     {
         int errcode = YAZ_BIB1_TEMPORARY_SYSTEM_ERROR;
-
-        wrbuf_destroy(json);
-        wrbuf_destroy(cf_file);
-
         return odr.create_searchResponse(req, errcode, e.what());
     }
     catch (CF_Engine_Error_Task &e)
@@ -948,25 +1077,24 @@ Z_APDU *yf::CF::Rep::open_cf_z3950(Z_APDU *req,  CF::ZSession *zsession,
         const char *addinfo = 0;
         int errcode = YAZ_BIB1_UNSPECIFIED_ERROR;
 
-        wrbuf_destroy(json);
-        wrbuf_destroy(cf_file);
-
         cf_interpret_assertions(&errcode, e.what(), &addinfo);
         return odr.create_searchResponse(req, errcode, addinfo);
     }
-    ses->metadata.parse_cf(wrbuf_cstr(cf_file));
-    wrbuf_destroy(json);
-    wrbuf_destroy(cf_file);
+    ses->metadata.parse_cf(cf_file.c_str());
     return 0;
 }
 
 Z_Records *yf::CF::Rep::fetch_z3950(
-    CF::ZSession *zsession,
-    mp::odr &odr, Odr_oid *preferredRecordSyntax,
-    Z_ElementSetNames *esn,
+    CF::ZSession *zsession, mp::odr &odr,
+    const Odr_oid *preferredRecordSyntax, const Z_ElementSetNames *esn,
     int start, int number, int &error_code, std::string &addinfo,
     int *number_returned, int *next_position)
 {
+    int details = 0;
+    if (esn && esn->which == Z_ElementSetNames_generic &&
+        *esn->u.generic == 'F')
+        details = 1;
+
     Z_Records *rec = (Z_Records *) odr_malloc(odr, sizeof(Z_Records));
     rec->which = Z_Records_DBOSD;
     rec->u.databaseOrSurDiagnostics = (Z_NamePlusRecordList *)
@@ -991,28 +1119,21 @@ Z_Records *yf::CF::Rep::fetch_z3950(
         /* if no record syntax was given assume XML */
         if (!oid || !oid_oidcmp(oid, yaz_oid_recsyn_xml))
         {
-            if (zsession->get_record_xml(start + i, wrb, &error_code))
-            {
-                npr->u.databaseRecord =
-                    z_ext_record_xml(odr, wrbuf_buf(wrb), wrbuf_len(wrb));
-            }
-            else
+            if (!zsession->get_record_xml(start + i, wrb, &error_code, details))
                 break;
+            npr->u.databaseRecord =
+                z_ext_record_xml(odr, wrbuf_buf(wrb), wrbuf_len(wrb));
         }
         else if (!oid_oidcmp(oid, yaz_oid_recsyn_json))
         {
             struct json_node *json =
-                zsession->get_record(start + i, &error_code);
-            if (json)
-            {
-                json_write_wrbuf(json, wrb);
-
-                npr->u.databaseRecord =
-                    z_ext_record_oid(odr, yaz_oid_recsyn_json,
-                                     wrbuf_buf(wrb), wrbuf_len(wrb));
-            }
-            else
+                zsession->get_record(start + i, &error_code, details);
+            if (!json)
                 break;
+            json_write_wrbuf(json, wrb);
+            npr->u.databaseRecord =
+                z_ext_record_oid(odr, yaz_oid_recsyn_json,
+                                 wrbuf_buf(wrb), wrbuf_len(wrb));
         }
         else
         { /* unsupported syntax. Return diagnostic */
@@ -1022,6 +1143,13 @@ Z_Records *yf::CF::Rep::fetch_z3950(
             break;
         }
     }
+    if (error_code == YAZ_BIB1_PRESENT_REQUEST_OUT_OF_RANGE &&
+        !addinfo.length())
+    {
+        char str[20];
+        sprintf(str, "%d", start + i);
+        addinfo = str;
+    }
     wrbuf_destroy(wrb);
     *next_position = start + i;
     *number_returned = i;
@@ -1058,13 +1186,24 @@ Z_APDU *yf::CF::Rep::search_z3950(Z_APDU *req, CF::ZSession *zsession,
         NMEM nmem = ((ODR) odr)->mem;
         struct sort_elem *sort_list = 0;
         type7_sort(sr->query->u.type_1, nmem, &sort_list);
-        if (!rpn_to_json.parse(sr->query->u.type_1,
-                zsession->subdatabase,sort_list))
+
+        if (!rpn_to_json.parse_full(sr->query->u.type_1,
+                                    zsession->subdatabase, sort_list))
         {
+            // Should succeed for al reasonable queries. If not, we can
+            // declare the whole search a failure.
             const char *addinfo;
             int errcode = rpn_to_json.get_diagnostic(&addinfo);
             return odr.create_searchResponse(req, errcode, addinfo);
         }
+        if (!rpn_to_json.parse(sr->query->u.type_1,
+                               zsession->subdatabase, sort_list))
+        {
+            const char *addinfo;
+            int errcode = rpn_to_json.get_diagnostic(&addinfo);
+            yaz_log(YLOG_DEBUG, "regular parse failed with %d '%s",
+                    errcode, addinfo );
+        }
     }
     else
     {
@@ -1072,24 +1211,39 @@ Z_APDU *yf::CF::Rep::search_z3950(Z_APDU *req, CF::ZSession *zsession,
     }
 
     const char *outp = 0;
-    int query_no; // pass over multiple JSON alternatives
-    for (query_no = 0; ; query_no++)
+    int errcode = 0;
+    char *errmsg = 0;
+
+    int query_no = -1; // pass over multiple JSON alternatives
+    for (;; query_no++)
     {
-        const char *json_res = rpn_to_json.get_result(query_no);
+        const char *json_res;
+
+        if (query_no == -1)
+            json_res = rpn_to_json.get_full();
+        else
+            json_res = rpn_to_json.get_result(query_no);
+
         if (!json_res)
         {
-            return odr.create_searchResponse(
-                req, YAZ_BIB1_UNSUPP_USE_ATTRIBUTE, 0);
+            if (errcode)
+                return odr.create_searchResponse(req, errcode, errmsg);
+            else
+            {
+                return odr.create_searchResponse(
+                    req, YAZ_BIB1_UNSUPP_USE_ATTRIBUTE, 0);
+            }
         }
         try
         {
             zsession->cf_session->engine->run_task("search", json_res, &outp);
+            break;
         }
         catch (CF_Engine_Error_System &e)
         {
-            const char *what = e.what();
-            return odr.create_searchResponse(req, YAZ_BIB1_TEMPORARY_SYSTEM_ERROR,
-                                             what);
+            return odr.create_searchResponse(req,
+                                             YAZ_BIB1_TEMPORARY_SYSTEM_ERROR,
+                                             e.what());
         }
         catch (CF_Engine_Error_Task &e)
         {
@@ -1097,29 +1251,28 @@ Z_APDU *yf::CF::Rep::search_z3950(Z_APDU *req, CF::ZSession *zsession,
             const char *what = e.what();
             json_res = rpn_to_json.get_result(1);
 
-            if (what && !strncmp(what, lead, strlen(lead)))
+            if (!what || strncmp(what, lead, strlen(lead)))
             {
-                continue; // try next query_no ..
-            }
-            else
-            {
-                return odr.create_searchResponse(
-                    req, YAZ_BIB1_TEMPORARY_SYSTEM_ERROR, what);
+                const char *addinfo = 0;
+                errcode = YAZ_BIB1_TEMPORARY_SYSTEM_ERROR;
+
+                cf_interpret_assertions(&errcode, what, &addinfo);
+                errmsg = odr_strdup_null(odr, addinfo);
             }
         }
-        break;
     }
 
-    const char *errmsg = 0;
-    struct json_node *res = json_parse(outp, &errmsg);
+    const char *json_errmsg = 0;
+    struct json_node *res = json_parse(outp, &json_errmsg);
     if (!res)
     {
         return odr.create_searchResponse(req, YAZ_BIB1_TEMPORARY_SYSTEM_ERROR,
-                                         errmsg);
+                                         json_errmsg);
     }
     struct json_node *hit = json_get_object(res, "hits");
     if (hit && hit->type == json_node_array)
         hit = json_get_elem(hit, 0);
+    bool estimated_hit_count = false;
     if (hit)
     {
         if (hit->type == json_node_number)
@@ -1127,6 +1280,12 @@ Z_APDU *yf::CF::Rep::search_z3950(Z_APDU *req, CF::ZSession *zsession,
         else if (hit->type == json_node_string)
             zsession->hits = odr_atoi(hit->u.string);
     }
+    else
+    {
+        zsession->hits = 100;
+        estimated_hit_count = true;
+    }
+
     json_remove_node(res);
 
     if (zsession->hits == 0)
@@ -1158,7 +1317,6 @@ Z_APDU *yf::CF::Rep::search_z3950(Z_APDU *req, CF::ZSession *zsession,
     }
     json_remove_node(res);
 
-
     int number = 0;
     mp::util::piggyback(*sr->smallSetUpperBound,
                         *sr->largeSetLowerBound,
@@ -1166,6 +1324,8 @@ Z_APDU *yf::CF::Rep::search_z3950(Z_APDU *req, CF::ZSession *zsession,
                         zsession->hits,
                         number);
 
+    std::string addinfo;
+    int error_code = 0;
     int number_returned = 0;
     int next_position = 0;
     Z_Records *records = 0;
@@ -1177,8 +1337,6 @@ Z_APDU *yf::CF::Rep::search_z3950(Z_APDU *req, CF::ZSession *zsession,
         else
             esn = sr->smallSetElementSetNames;
 
-        std::string addinfo;
-        int error_code = 0;
         records = fetch_z3950(
             zsession,
             odr, sr->preferredRecordSyntax, esn,
@@ -1187,12 +1345,16 @@ Z_APDU *yf::CF::Rep::search_z3950(Z_APDU *req, CF::ZSession *zsession,
             &number_returned,
             &next_position);
     }
-    Z_APDU *apdu_res = odr.create_searchResponse(req, 0, 0);
-    *apdu_res->u.searchResponse->resultCount = zsession->hits;
-    apdu_res->u.searchResponse->records = records;
+    Z_APDU *apdu_res = odr.create_searchResponse(
+        req, error_code, addinfo.length() ? addinfo.c_str() : 0);
+    Z_SearchResponse *resp = apdu_res->u.searchResponse;
+    *resp->resultCount = zsession->hits;
+    if (estimated_hit_count)
+        resp->resultSetStatus = odr_intdup(odr, Z_SearchResponse_estimate);
 
-    *apdu_res->u.searchResponse->numberOfRecordsReturned = number_returned;
-    *apdu_res->u.searchResponse->nextResultSetPosition = next_position;
+    resp->records = records;
+    *resp->numberOfRecordsReturned = number_returned;
+    *resp->nextResultSetPosition = next_position;
 
     return apdu_res;
 }
